package l13_lambda;

import java.util.List;

//	'forEach' is a new method added to the List interface in Java 8
//	Double Colon operator (Java 8) is used as method reference.

public class DoubleColon {

	record MyString(String string) {

		public void print() {
			System.out.println("print(): " + string);
		}

		@Override
		public String toString() {
			return "toString(): " + string;
		}
	}

	private static void printStatic(String name){
		System.out.println("printStatic() : " + name);
	}

	public static void main(String[] args) {

		List<String> names = List.of("one", "two", "three");

		// For class methods, container element is the method parameter
//		for (String name : names) DoubleColon.printStatic(name);
		names.forEach(DoubleColon::printStatic);
		System.out.println();

		// Instance method (println) qualified by an object (System.out).
		// Container element is the function argument
		// (if object::functionName syntax is used)
//		for (String name : names) System.out.println(name);
		names.forEach(System.out::println);
		System.out.println();

		List<MyString> myStrings = List.of(new MyString("string 1"), new MyString("string 2"));

		// For instance methods container element is the method qualifier
		// (if className::functionName syntax is used)
//		for (MyString myString : myStrings) myString.print();
		myStrings.forEach(MyString::print);
		System.out.println();

//		for (MyString myString : myStrings) System.out.println(myString);
		myStrings.forEach(System.out::println);
		System.out.println();
	}
}

