package l13_lambda;

/*
	A lambda expression is composed of three parts.
	Argument List 	Arrow Token 	Body
	(int x, int y) 	->				x + y

	The body can be either a single expression or a statement block. In the
	expression form, the body is simply evaluated and returned.
*/

public class LambdaTester {

	@FunctionalInterface
	interface MathOperation {
		int operation(int a, int b);
	}

	private static int operate(MathOperation mathOperation, int a, int b) {
		return mathOperation.operation(a, b);
	}

	public static void main(String[] args) {

		// with type declaration
		MathOperation addition = (int a, int b) -> a + b;

		// without type declaration
		MathOperation subtraction = (a, b) -> a - b;

		// with return statement along with curly braces
		MathOperation multiplication = (a, b) -> { return a * b; };

		System.out.println("10 + 5 = " + operate(addition, 10, 5));
		System.out.println("20 - 8 = " + operate(subtraction, 20, 8));
		System.out.println("10 x 5 = " + operate(multiplication, 10, 5));
		// directly in 'operate' call
		System.out.println("10 / 5 = " + operate((a, b) -> a / b, 10, 5));
	}
}

