package l13_lambda;

import java.util.Arrays;
import java.util.List;
import java.util.function.Predicate;

//	In this example we use Predicate functional interface to create a test and
//	print the elements that pass the test. This way you can provide the logic
//	using lambda expression and do something based on it.

//	@FunctionalInterface
//	public interface Predicate<T> { boolean test(T t); }

//	Represents a predicate (boolean-valued function) of one argument.

//	This is a functional interface whose functional method is test(T).

public class PredicateGenericTest {

	private static <E> void evaluate(List<E> list, Predicate<E> predicate, String msg) {
		System.out.print("Print " + msg + ":");
		for(E n : list) {
			if (predicate.test(n)) System.out.print(" " + n);
		}
		System.out.println();
	}

	public static void main(String [] a) {

		List<Integer> integerList = Arrays.asList(1, 2, 3, 4, 5, 6, 7);
		evaluate(integerList, n -> true, "all numbers");
		evaluate(integerList, n -> false, "no numbers");
		evaluate(integerList, n -> n % 2 == 0, "even numbers");
		evaluate(integerList, n -> n % 2 == 1, "odd numbers");
		evaluate(integerList, n -> n > 5, "numbers > 5");
		System.out.println();

		List<String> stringList = List.of("This", "is", "string", "list");
		evaluate(stringList, s -> s.startsWith("s"), "strings starting with 's'");
		evaluate(stringList, s -> s.length() == 4, "strings of length 4");
	}
}

